@@ -1,4 +1,5 @@
 #include "gcab-priv.h"
+#include <glib/gi18n-lib.h>
 
 static voidpf
 zalloc (voidpf opaque, uInt items, uInt size)
@@ -15,10 +16,15 @@ zfree (voidpf opaque, voidpf address)
 static gboolean
 cdata_set (cdata_t *cd, int type, guint8 *data, size_t size)
 {
+    if (type > GCAB_COMPRESSION_MSZIP) {
+        g_critical (_("unsupported compression method %d"), type);
+        return FALSE;
+    }
+
     cd->nubytes = size;
 
     if (type == 0) {
-        memcpy (cd->data, data, size);
+        memcpy (cd->in, data, size);
         cd->ncbytes = size;
     }
 
@@ -31,11 +37,11 @@ cdata_set (cdata_t *cd, int type, guint8 *data, size_t size)
             return FALSE;
         stream.next_in = data;
         stream.avail_in = size;
-        stream.next_out = cd->data + 2;
-        stream.avail_out = sizeof (cd->data) - 2;
+        stream.next_out = cd->in + 2;
+        stream.avail_out = sizeof (cd->in) - 2;
         /* insert the signature */
-        cd->data[0] = 'C';
-        cd->data[1] = 'K';
+        cd->in[0] = 'C';
+        cd->in[1] = 'K';
         deflate (&stream, Z_FINISH);
         deflateEnd (&stream);
         cd->ncbytes = stream.total_out + 2;
@@ -385,7 +391,7 @@ cdata_write (cdata_t *cd, GDataOutputStream *out, int type,
     if (!cdata_set(cd, type, data, size))
         return FALSE;
 
-    CHECKSUM datacsum = compute_checksum(cd->data, cd->ncbytes, 0);
+    CHECKSUM datacsum = compute_checksum(cd->in, cd->ncbytes, 0);
     cd->checksum = compute_checksum ((guint8*)&cd->ncbytes, 4, datacsum);
     GOutputStream *stream = g_filter_output_stream_get_base_stream (G_FILTER_OUTPUT_STREAM (out));
 
@@ -394,7 +400,7 @@ cdata_write (cdata_t *cd, GDataOutputStream *out, int type,
     if ((!W4 (cd->checksum)) ||
         (!W2 (cd->ncbytes)) ||
         (!W2 (cd->nubytes)) ||
-        (g_output_stream_write (stream, cd->data, cd->ncbytes, cancellable, error) == -1))
+        (g_output_stream_write (stream, cd->in, cd->ncbytes, cancellable, error) == -1))
         return FALSE;
 
     *bytes_written = 4 + 2 + 2 + cd->ncbytes;
@@ -420,22 +426,30 @@ cdata_finish (cdata_t *cd, GError **error)
 }
 
 G_GNUC_INTERNAL gboolean
-cdata_read (cdata_t *cd, u1 res_data, GCabCompression compression,
+cdata_read (cdata_t *cd, u1 res_data, gint comptype,
             GDataInputStream *in, GCancellable *cancellable, GError **error)
 
 {
     gboolean success = FALSE;
-    int zret = Z_OK;
-    gchar *data = compression == GCAB_COMPRESSION_NONE ? cd->out : cd->data;
+    int ret, zret = Z_OK;
+    gint compression = comptype & GCAB_COMPRESSION_MASK;
+    gchar *buf = compression == GCAB_COMPRESSION_NONE ? cd->out : cd->in;
+
+    if (compression > GCAB_COMPRESSION_MSZIP &&
+        compression != GCAB_COMPRESSION_LZX) {
+        g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
+                     _("unsupported compression method %d"), compression);
+        return FALSE;
+    }
 
     R4 (cd->checksum);
     R2 (cd->ncbytes);
     R2 (cd->nubytes);
     cd->reserved = g_malloc (res_data);
     RN (cd->reserved, res_data);
-    RN (data, cd->ncbytes);
+    RN (buf, cd->ncbytes);
 
-    CHECKSUM datacsum = compute_checksum(data, cd->ncbytes, 0);
+    CHECKSUM datacsum = compute_checksum(buf, cd->ncbytes, 0);
     g_return_val_if_fail (cd->checksum == compute_checksum ((guint8*)&cd->ncbytes, 4, datacsum), FALSE);
 
     if (g_getenv ("GCAB_DEBUG")) {
@@ -445,39 +459,57 @@ cdata_read (cdata_t *cd, u1 res_data, GCabCompression compression,
         P2 (cd, nubytes);
         if (res_data)
             PN (cd, reserved, res_data);
-        PND (cd, data, 64);
+        PND (cd, buf, 64);
+    }
+
+    if (compression == GCAB_COMPRESSION_LZX) {
+        if (cd->fdi.alloc == NULL) {
+            cd->fdi.alloc = g_malloc;
+            cd->fdi.free = g_free;
+            cd->decomp.fdi = &cd->fdi;
+            cd->decomp.inbuf = cd->in;
+            cd->decomp.outbuf = cd->out;
+
+            ret = LZXfdi_init((comptype >> 8) & 0x1f, &cd->decomp);
+            if (ret < 0)
+                goto end;
+        }
+
+        ret = LZXfdi_decomp (cd->ncbytes, cd->nubytes, &cd->decomp);
+        if (ret < 0)
+            goto end;
     }
 
     if (compression == GCAB_COMPRESSION_MSZIP) {
-        z_stream *z = &cd->z;
-
-        if (cd->data[0] != 'C' || cd->data[1] != 'K')
+        if (cd->in[0] != 'C' || cd->in[1] != 'K')
             goto end;
 
+        z_stream *z = &cd->z;
+
         z->avail_in = cd->ncbytes - 2;
-        z->next_in = cd->data + 2;
+        z->next_in = cd->in + 2;
         z->avail_out = cd->nubytes;
         z->next_out = cd->out;
+        z->total_out = 0;
 
         if (!z->opaque) {
             z->zalloc = zalloc;
             z->zfree = zfree;
             z->opaque = cd;
 
-            zret = inflateInit2 (z, -15);
+            zret = inflateInit2 (z, -MAX_WBITS);
             if (zret != Z_OK)
                 goto end;
         }
 
         while (1) {
             zret = inflate (z, Z_BLOCK);
-            if (zret != Z_OK)
+            if (zret == Z_STREAM_END)
                 break;
+            if (zret != Z_OK)
+                goto end;
         }
 
-        if (zret != Z_STREAM_END)
-            goto end;
-
         g_warn_if_fail (z->avail_in == 0);
         g_warn_if_fail (z->avail_out == 0);
         if (z->avail_in != 0 || z->avail_out != 0)
@@ -498,9 +530,10 @@ end:
     if (zret != Z_OK)
         g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
                      "zlib failed: %s", zError (zret));
+
     if (!*error && !success)
         g_set_error (error, GCAB_ERROR, GCAB_ERROR_FAILED,
-                     "Invalid cabint chunk");
+                     "Invalid cabinet chunk");
 
     return success;
 }
